@@ -81,6 +81,7 @@ NOEXPORT int change_root(void);
 #endif
 NOEXPORT int pipe_init(SOCKET [2], char *);
 NOEXPORT int signal_pipe_dispatch(void);
+NOEXPORT void reload_config();
 NOEXPORT int process_connections(void);
 NOEXPORT char *signal_name(int);
 
@@ -808,39 +809,7 @@ NOEXPORT int signal_pipe_dispatch(void) {
         return 1;
     case SIGNAL_RELOAD_CONFIG:
         s_log(LOG_DEBUG, "Processing SIGNAL_RELOAD_CONFIG");
-        if(options_parse(CONF_RELOAD)) {
-            s_log(LOG_ERR, "Failed to reload the configuration file");
-        } else {
-#ifdef HAVE_CHROOT
-            struct stat sb;
-#endif /* HAVE_CHROOT */
-            unbind_ports();
-            log_flush(LOG_MODE_BUFFER);
-#ifdef HAVE_CHROOT
-            /* we don't close SINK_SYSLOG if chroot is enabled and
-             * there is no /dev/log inside it, which could allow
-             * openlog(3) to reopen the syslog socket later */
-            if(global_options.chroot_dir && stat("/dev/log", &sb))
-                log_close(SINK_OUTFILE);
-            else
-#endif /* HAVE_CHROOT */
-                log_close(SINK_SYSLOG|SINK_OUTFILE);
-            /* there is no race condition here:
-             * client threads are not allowed to use global options */
-            options_free();
-            options_apply();
-            /* we hope that a sane openlog(3) implementation won't
-             * attempt to reopen /dev/log if it's already open */
-            log_open(SINK_SYSLOG|SINK_OUTFILE);
-            log_flush(LOG_MODE_CONFIGURED);
-            ui_config_reloaded();
-            if(bind_ports()) {
-                /* FIXME: handle the error */
-            }
-            if(exec_connect_start()) {
-                /* FIXME: handle the error */
-            }
-        }
+        reload_config();
         return 0;
     case SIGNAL_REOPEN_LOG:
         s_log(LOG_DEBUG, "Processing SIGNAL_REOPEN_LOG");
@@ -860,6 +829,48 @@ NOEXPORT int signal_pipe_dispatch(void) {
     }
 }
 
+NOEXPORT void reload_config() {
+    static int delay=10; /* 10ms */
+#ifdef HAVE_CHROOT
+    struct stat sb;
+#endif /* HAVE_CHROOT */
+
+    if(options_parse(CONF_RELOAD)) {
+        s_log(LOG_ERR, "Failed to reload the configuration file");
+        return;
+    }
+    unbind_ports();
+    log_flush(LOG_MODE_BUFFER);
+#ifdef HAVE_CHROOT
+    /* we don't close SINK_SYSLOG if chroot is enabled and
+     * there is no /dev/log inside it, which could allow
+     * openlog(3) to reopen the syslog socket later */
+    if(global_options.chroot_dir && stat("/dev/log", &sb))
+        log_close(SINK_OUTFILE);
+    else
+#endif /* HAVE_CHROOT */
+        log_close(SINK_SYSLOG|SINK_OUTFILE);
+    /* there is no race condition here:
+     * client threads are not allowed to use global options */
+    options_free();
+    options_apply();
+    /* we hope that a sane openlog(3) implementation won't
+     * attempt to reopen /dev/log if it's already open */
+    log_open(SINK_SYSLOG|SINK_OUTFILE);
+    log_flush(LOG_MODE_CONFIGURED);
+    ui_config_reloaded();
+    /* we use "|" instead of "||" to attempt initialization of both subsystems */
+    if(bind_ports() | exec_connect_start()) {
+        s_poll_sleep(delay/1000, delay%1000); /* sleep to avoid log trashing */
+        signal_post(SIGNAL_RELOAD_CONFIG); /* retry */
+        delay*=2;
+        if(delay > 10000) /* 10s */
+            delay=10000;
+    } else {
+        delay=10; /* 10ms */
+    }
+}
+
 #ifdef __GNUC__
 #if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
 #pragma GCC diagnostic push
